#!/usr/bin/env python
#
# Constants (derived from the mwalib rust public constants)
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

#
# the velocity factor of electic fields in RG-6 like coax cable
#
MWA_COAX_V_FACTOR = 1.204

# The MWA's altitude in metres.
MWA_ALTITUDE_METRES = 377.827

# The MWA's latitude on Earth in radians. This is -26d42m11.94986s.
MWA_LATITUDE_RADIANS = -0.4660608448386394

# The MWA's longitude on Earth in radians. This is 116d40m14.93485s.
MWA_LONGITUDE_RADIANS = 2.0362898668561042